@@ -13,7 +13,6 @@ import tempfile
 from glob import glob
 import json as simplejson
 
-import urllib3
 import background
 import click
 import click_completion
@@ -27,14 +26,17 @@ import pipdeptree
 import semver
 from pipreqs import pipreqs
 from blindspin import spinner
-from urllib3.exceptions import InsecureRequestWarning
+
+from requests.packages import urllib3
+from requests.packages.urllib3.exceptions import InsecureRequestWarning
+
 from .project import Project
 from .utils import (
     convert_deps_from_pip, convert_deps_to_pip, is_required_version,
-    proper_case, pep423_name, split_file, merge_deps, resolve_deps, shellquote, is_vcs,
+    proper_case, pep423_name, split_file, merge_deps, venv_resolve_deps, shellquote, is_vcs,
     python_version, find_windows_executable, is_file, prepare_pip_source_args,
     temp_environ, is_valid_url, download_file, get_requirement, need_update_check,
-    touch_update_stamp
+    touch_update_stamp, is_pinned, is_star, TemporaryDirectory
 )
 from .__version__ import __version__
 from . import pep508checker, progress
@@ -54,7 +56,6 @@ if sys.version_info < (3, 3):
 else:
     from shutil import get_terminal_size
 
-
 # Packages that should be ignored later.
 BAD_PACKAGES = (
     'setuptools', 'pip', 'wheel', 'six', 'packaging', 'distribute'
@@ -98,10 +99,28 @@ if PIPENV_NOSPIN:
     def spinner():
         yield
 
-# Disable warnings for Python 2.6.
-urllib3.disable_warnings(InsecureRequestWarning)
 
-project = Project()
+def which(command, location=None, allow_global=False):
+    if location is None:
+        location = project.virtualenv_location
+
+    if not allow_global:
+        if os.name == 'nt':
+            p = find_windows_executable(os.path.join(location, 'Scripts'), command)
+        else:
+            p = os.sep.join([location] + ['bin/{0}'.format(command)])
+    else:
+        if command == 'python':
+            p = sys.executable
+
+    return p
+
+
+# Disable warnings for Python 2.6.
+if 'urllib3' in globals():
+    urllib3.disable_warnings(InsecureRequestWarning)
+
+project = Project(which=which)
 
 
 def load_dot_env():
@@ -720,7 +739,7 @@ def do_where(virtualenv=False, bare=True):
 
 def do_install_dependencies(
     dev=False, only=False, bare=False, requirements=False, allow_global=False,
-    ignore_hashes=False, skip_lock=False, verbose=False, concurrent=True
+    ignore_hashes=False, skip_lock=False, verbose=False, concurrent=True, requirements_dir=None
 ):
     """"Executes the install functionality.
 
@@ -800,17 +819,16 @@ def do_install_dependencies(
         for l in (deps_list, dev_deps_list):
             for i, dep in enumerate(l):
                 if '--hash' not in dep[0]:
-                    l[i] = list(l[i])
-                    l[i][0] = '# {0}'.format(l[i][0])
+                    l[i] = ('# {0}'.format(l[i][0]),) + l[i][1:]
 
         # Output only default dependencies
         if not dev:
-            click.echo('\n'.join(d[0] for d in deps_list))
+            click.echo('\n'.join(d[0] for d in sorted(deps_list)))
             sys.exit(0)
 
         # Output only dev dependencies
         if dev:
-            click.echo('\n'.join(d[0] for d in dev_deps_list))
+            click.echo('\n'.join(d[0] for d in sorted(dev_deps_list)))
             sys.exit(0)
 
     procs = []
@@ -834,7 +852,8 @@ def do_install_dependencies(
                 no_deps=no_deps,
                 verbose=verbose,
                 block=block,
-                index=index
+                index=index,
+                requirements_dir=requirements_dir
             )
 
             c.dep = dep
@@ -867,7 +886,8 @@ def do_install_dependencies(
                 allow_global=allow_global,
                 no_deps=no_deps,
                 verbose=verbose,
-                index=index
+                index=index,
+                requirements_dir=requirements_dir
             )
 
             # The Installation failed...
@@ -1009,20 +1029,30 @@ def get_downloads_info(names_map, section):
     return info
 
 
-def do_lock(verbose=False, system=False, clear=False, pre=False):
+def do_lock(verbose=False, system=False, clear=False, pre=False, keep_outdated=False, write=True):
     """Executes the freeze functionality."""
 
+    cached_lockfile = {}
+    if keep_outdated:
+        if not project.lockfile_exists:
+            click.echo('{0}: Pipfile.lock must exist to use --keep-outdated!'.format(
+                crayons.red('Warning', bold=True)
+            ))
+            sys.exit(1)
+        cached_lockfile = project.lockfile_content
+
     project.destroy_lockfile()
 
-    # Alert the user of progress.
-    click.echo(
-        u'{0} {1} {2}'.format(
-            crayons.normal('Locking'),
-            crayons.red('[dev-packages]'),
-            crayons.normal('dependencies…')
-        ),
-        err=True
-    )
+    if write:
+        # Alert the user of progress.
+        click.echo(
+            u'{0} {1} {2}'.format(
+                crayons.normal('Locking'),
+                crayons.red('[dev-packages]'),
+                crayons.normal('dependencies…')
+            ),
+            err=True
+        )
 
     # Create the lockfile.
     lockfile = project._lockfile
@@ -1033,19 +1063,23 @@ def do_lock(verbose=False, system=False, clear=False, pre=False):
             if not hasattr(v, 'keys'):
                 del lockfile[section][k]
 
+    # Ensure that develop inherits from default.
+    dev_packages = project.dev_packages.copy()
+
+    for dev_package in project.dev_packages:
+        if dev_package in project.packages:
+            dev_packages[dev_package] = project.packages[dev_package]
+
     # Resolve dev-package dependencies, with pip-tools.
-    deps = convert_deps_to_pip(project.dev_packages, project, r=False, include_index=True)
-    results = resolve_deps(
+    deps = convert_deps_to_pip(dev_packages, project, r=False, include_index=True)
+
+    results = venv_resolve_deps(
         deps,
-        sources=project.sources,
-        verbose=verbose,
-        python=python_version(which('python', allow_global=system)),
-        clear=clear,
         which=which,
-        which_pip=which_pip,
+        verbose=verbose,
         project=project,
+        clear=clear,
         pre=pre,
-        allow_global=system
     )
 
     # Add develop dependencies to lockfile.
@@ -1054,7 +1088,7 @@ def do_lock(verbose=False, system=False, clear=False, pre=False):
         lockfile['develop'].update({dep['name']: {'version': '=={0}'.format(dep['version'])}})
 
         # Add Hashes to lockfile
-        lockfile['develop'][dep['name']]['hashes'] = dep['hashes']
+        lockfile['develop'][dep['name']]['hashes'] = sorted(dep['hashes'])
 
         # Add index metadata to lockfile.
         if 'index' in dep:
@@ -1086,28 +1120,26 @@ def do_lock(verbose=False, system=False, clear=False, pre=False):
             except IndexError:
                 pass
 
-    # Alert the user of progress.
-    click.echo(
-        u'{0} {1} {2}'.format(
-            crayons.normal('Locking'),
-            crayons.red('[packages]'),
-            crayons.normal('dependencies…')
-        ),
-        err=True
-    )
+    if write:
+        # Alert the user of progress.
+        click.echo(
+            u'{0} {1} {2}'.format(
+                crayons.normal('Locking'),
+                crayons.red('[packages]'),
+                crayons.normal('dependencies…')
+            ),
+            err=True
+        )
 
     # Resolve package dependencies, with pip-tools.
     deps = convert_deps_to_pip(project.packages, project, r=False, include_index=True)
-    results = resolve_deps(
+    results = venv_resolve_deps(
         deps,
-        sources=project.sources,
-        verbose=verbose,
-        python=python_version(which('python', allow_global=system)),
         which=which,
-        which_pip=which_pip,
+        verbose=verbose,
         project=project,
+        clear=clear,
         pre=pre,
-        allow_global=system
     )
 
     # Add default dependencies to lockfile.
@@ -1116,7 +1148,7 @@ def do_lock(verbose=False, system=False, clear=False, pre=False):
         lockfile['default'].update({dep['name']: {'version': '=={0}'.format(dep['version'])}})
 
         # Add Hashes to lockfile
-        lockfile['default'][dep['name']]['hashes'] = dep['hashes']
+        lockfile['default'][dep['name']]['hashes'] = sorted(dep['hashes'])
 
         # Add index metadata to lockfile.
         if 'index' in dep:
@@ -1138,62 +1170,72 @@ def do_lock(verbose=False, system=False, clear=False, pre=False):
                 name = list(installed.keys())[0]
 
                 if is_vcs(installed[name]):
-                    # Convert name to PEP 432 name.
+                    # Convert name to PEP 423 name.
                     installed = {pep423_name(name): installed[name]}
 
                     lockfile['default'].update(installed)
             except IndexError:
                 pass
 
-    # Run the PEP 508 checker in the virtualenv, add it to the lockfile.
-    cmd = '"{0}" {1}'.format(which('python', allow_global=system), shellquote(pep508checker.__file__.rstrip('cdo')))
-    c = delegator.run(cmd)
-    try:
-        lockfile['_meta']['host-environment-markers'] = simplejson.loads(c.out)
-    except ValueError:
-        click.echo(crayons.red("An unexpected error occurred while accessing your virtualenv's python installation!"))
-        click.echo('Please run $ {0} to re-create your environment.'.format(crayons.red('pipenv --rm')))
-        sys.exit(1)
+    # Support for --keep-outdated…
+    if keep_outdated:
+        for section_name, section in (('default', project.packages), ('develop', project.dev_packages)):
+            for package_specified in section:
+                norm_name = pep423_name(package_specified)
+                if not is_pinned(section[package_specified]):
+                    lockfile[section_name][norm_name] = cached_lockfile[section_name][norm_name]
 
-    # Write out the lockfile.
-    with open(project.lockfile_location, 'w') as f:
-        simplejson.dump(lockfile, f, indent=4, separators=(',', ': '), sort_keys=True)
-        # Write newline at end of document. GH Issue #319.
-        f.write('\n')
+    # Overwrite any develop packages with default packages.
+    for default_package in lockfile['default']:
+        if default_package in lockfile['develop']:
+            lockfile['develop'][default_package] = lockfile['default'][default_package]
 
-    click.echo(
-        '{0}'.format(
-            crayons.normal(
-                'Updated Pipfile.lock ({0})!'.format(
-                    lockfile['_meta'].get('hash', {}).get('sha256')[-6:]
-                ),
-                bold=True
-            )
-        ),
-        err=True
-    )
+    if write:
+
+        # Write out the lockfile.
+        with open(project.lockfile_location, 'w') as f:
+            simplejson.dump(lockfile, f, indent=4, separators=(',', ': '), sort_keys=True)
+            # Write newline at end of document. GH Issue #319.
+            f.write('\n')
+
+        click.echo(
+            '{0}'.format(
+                crayons.normal(
+                    'Updated Pipfile.lock ({0})!'.format(
+                        lockfile['_meta'].get('hash', {}).get('sha256')[-6:]
+                    ),
+                    bold=True
+                )
+            ),
+            err=True
+        )
+    else:
+        return lockfile
 
 
 def activate_virtualenv(source=True):
     """Returns the string to activate a virtualenv."""
 
-    # Suffix for other shells.
+    # Suffix and source command for other shells.
     suffix = ''
+    command = '.' if source else ''
 
     # Support for fish shell.
     if PIPENV_SHELL and 'fish' in PIPENV_SHELL:
         suffix = '.fish'
+        command = 'source'
 
     # Support for csh shell.
     if PIPENV_SHELL and 'csh' in PIPENV_SHELL:
         suffix = '.csh'
+        command = 'source'
 
     # Escape any spaces located within the virtualenv path to allow
     # for proper activation.
     venv_location = project.virtualenv_location.replace(' ', r'\ ')
 
     if source:
-        return '. {0}/bin/activate{1}'.format(venv_location, suffix)
+        return '{2} {0}/bin/activate{1}'.format(venv_location, suffix, command)
     else:
         return '{0}/bin/activate'.format(venv_location)
 
@@ -1261,7 +1303,8 @@ def do_purge(bare=False, downloads=False, allow_global=False, verbose=False):
 
 def do_init(
     dev=False, requirements=False, allow_global=False, ignore_pipfile=False,
-    skip_lock=False, verbose=False, system=False, concurrent=True, deploy=False, pre=False
+    skip_lock=False, verbose=False, system=False, concurrent=True, deploy=False,
+    pre=False, keep_outdated=False, requirements_dir=None
 ):
     """Executes the init functionality."""
 
@@ -1276,6 +1319,9 @@ def do_init(
     # Ensure the Pipfile exists.
     ensure_pipfile()
 
+    if not requirements_dir:
+        requirements_dir = TemporaryDirectory(suffix='-requirements', prefix='pipenv-')
+
     # Write out the lockfile if it doesn't exist, but not if the Pipfile is being ignored
     if (project.lockfile_exists and not ignore_pipfile) and not skip_lock:
 
@@ -1301,6 +1347,7 @@ def do_init(
                     )
                 )
                 click.echo(crayons.normal('Aborting deploy.', bold=True), err=True)
+                requirements_dir.cleanup()
                 sys.exit(1)
             else:
                 click.echo(
@@ -1318,10 +1365,12 @@ def do_init(
     # Write out the lockfile if it doesn't exist.
     if not project.lockfile_exists and not skip_lock:
         click.echo(crayons.normal(u'Pipfile.lock not found, creating…', bold=True), err=True)
-        do_lock(system=system, pre=pre)
+        do_lock(system=system, pre=pre, keep_outdated=keep_outdated, verbose=verbose)
 
     do_install_dependencies(dev=dev, requirements=requirements, allow_global=allow_global,
-                            skip_lock=skip_lock, verbose=verbose, concurrent=concurrent)
+                            skip_lock=skip_lock, verbose=verbose, concurrent=concurrent,
+                            requirements_dir=requirements_dir.name)
+    requirements_dir.cleanup()
 
     # Activate virtualenv instructions.
     if not allow_global and not deploy:
@@ -1330,7 +1379,8 @@ def do_init(
 
 def pip_install(
     package_name=None, r=None, allow_global=False, ignore_hashes=False,
-    no_deps=True, verbose=False, block=True, index=None, pre=False
+    no_deps=True, verbose=False, block=True, index=None, pre=False,
+    selective_upgrade=False, requirements_dir=None
 ):
     import pip
 
@@ -1339,9 +1389,9 @@ def pip_install(
         pip.logger.setLevel(logging.INFO)
 
     # Create files for hash mode.
-    if (not ignore_hashes) and (r is None):
-        r = tempfile.mkstemp(prefix='pipenv-', suffix='-requirement.txt')[1]
-        with open(r, 'w') as f:
+    if not package_name.startswith('-e ') and (not ignore_hashes) and (r is None):
+        fd, r = tempfile.mkstemp(prefix='pipenv-', suffix='-requirement.txt', dir=requirements_dir)
+        with os.fdopen(fd, 'w') as f:
             f.write(package_name)
 
     # Install dependencies when a package is a VCS dependency.
@@ -1378,10 +1428,10 @@ def pip_install(
         sources = project.sources
 
     for source in sources:
-        if r:
-            install_reqs = ' -r {0}'.format(r)
-        elif package_name.startswith('-e '):
+        if package_name.startswith('-e '):
             install_reqs = ' -e "{0}"'.format(package_name.split('-e ')[1])
+        elif r:
+            install_reqs = ' -r {0}'.format(r)
         else:
             install_reqs = ' "{0}"'.format(package_name)
 
@@ -1404,15 +1454,17 @@ def pip_install(
 
         quoted_pip = which_pip(allow_global=allow_global)
         quoted_pip = shellquote(quoted_pip)
+        upgrade_strategy = '--upgrade-strategy=only-if-needed' if selective_upgrade else ''
 
-        pip_command = '{0} install {4} {5} {6} {3} {1} {2} --exists-action w'.format(
+        pip_command = '{0} install {4} {5} {6} {7} {3} {1} {2} --exists-action w'.format(
             quoted_pip,
             install_reqs,
             ' '.join(prepare_pip_source_args([source])),
             no_deps,
             pre,
             src,
-            verbose_flag
+            verbose_flag,
+            upgrade_strategy
         )
 
         if verbose:
@@ -1440,22 +1492,6 @@ def pip_download(package_name):
     return c
 
 
-def which(command, location=None, allow_global=False):
-    if location is None:
-        location = project.virtualenv_location
-
-    if not allow_global:
-        if os.name == 'nt':
-            p = find_windows_executable(os.path.join(location, 'Scripts'), command)
-        else:
-            p = os.sep.join([location] + ['bin/{0}'.format(command)])
-    else:
-        if command == 'python':
-            p = sys.executable
-
-    return p
-
-
 def which_pip(allow_global=False):
     """Returns the location of virtualenv-installed pip."""
     if allow_global:
@@ -1514,6 +1550,7 @@ def format_help(help):
     help = help.replace('  shell', str(crayons.yellow('  shell', bold=True)))
     help = help.replace('  sync', str(crayons.green('  sync', bold=True)))
     help = help.replace('  uninstall', str(crayons.magenta('  uninstall', bold=True)))
+    help = help.replace('  update', str(crayons.green('  update', bold=True)))
 
     additional_help = """
 Usage Examples:
@@ -1596,9 +1633,11 @@ def warn_in_virtualenv():
             )
 
 
-def ensure_lockfile():
+def ensure_lockfile(keep_outdated=False):
     """Ensures that the lockfile is up–to–date."""
     pre = project.settings.get('allow_prereleases')
+    if not keep_outdated:
+        keep_outdated = project.settings.get('keep_outdated')
 
     # Write out the lockfile if it doesn't exist, but not if the Pipfile is being ignored
     if project.lockfile_exists:
@@ -1626,7 +1665,9 @@ def ensure_lockfile():
                 err=True
             )
 
-            do_lock(pre=pre)
+            do_lock(pre=pre, keep_outdated=keep_outdated)
+    else:
+        do_lock(pre=pre, keep_outdated=keep_outdated)
 
 
 def do_py(system=False):
@@ -1635,16 +1676,51 @@ def do_py(system=False):
     except AttributeError:
         click.echo(crayons.red('No project found!'))
 
+def do_outdated():
+    packages = {}
+    results = delegator.run('{0} freeze'.format(which('pip'))).out.strip().split('\n')
+    for result in results:
+        packages.update(convert_deps_from_pip(result))
+
+    updated_packages = {}
+
+    lockfile = do_lock(write=False)
+    for section in ('develop', 'default'):
+        for package in lockfile[section]:
+            try:
+                updated_packages[package] = lockfile[section][package]['version']
+            except KeyError:
+                pass
+
+    outdated = []
+    for package in packages:
+        if package in updated_packages:
+            if updated_packages[package] != packages[package]:
+                outdated.append((package, updated_packages[package], packages[package]))
+
+    for package, new_version, old_version in outdated:
+        click.echo('Package {0!r} out–of–date: {1!r} installed, {2!r} available.'.format(package, old_version, new_version))
+
+    sys.exit(bool(outdated))
+
 
 
 def do_install(
     package_name=False, more_packages=False, dev=False, three=False,
     python=False, system=False, lock=True, ignore_pipfile=False,
     skip_lock=False, verbose=False, requirements=False, sequential=False,
-    pre=False, code=False, deploy=False
+    pre=False, code=False, deploy=False, keep_outdated=False,
+    selective_upgrade=False
 ):
     import pip
 
+    requirements_directory = TemporaryDirectory(suffix='-requirements', prefix='pipenv-')
+    if selective_upgrade:
+        keep_outdated = True
+
+    if not more_packages:
+        more_packages = []
+
     # Don't search for requirements.txt files if the user provides one
     skip_requirements = True if requirements else False
 
@@ -1657,6 +1733,9 @@ def do_install(
     if not pre:
         pre = project.settings.get('allow_prereleases')
 
+    if not keep_outdated:
+        keep_outdated = project.settings.get('keep_outdated')
+
     remote = requirements and is_valid_url(requirements)
 
     # Warn and exit if --system is used without a pipfile.
@@ -1668,6 +1747,7 @@ def do_install(
             ), err=True
         )
         click.echo('See also: --deploy flag.', err=True)
+        requirements_directory.cleanup()
         sys.exit(1)
 
     # Automatically use an activated virtualenv.
@@ -1676,7 +1756,7 @@ def do_install(
 
     # Check if the file is remote or not
     if remote:
-        fd, temp_reqs = tempfile.mkstemp(prefix='pipenv-', suffix='-requirement.txt')
+        fd, temp_reqs = tempfile.mkstemp(prefix='pipenv-', suffix='-requirement.txt', dir=requirements_directory.name)
         requirements_url = requirements
 
         # Download requirements file
@@ -1690,6 +1770,7 @@ def do_install(
                 ),
                 err=True
             )
+            requirements_directory.cleanup()
             sys.exit(1)
         # Replace the url with the temporary requirements file
         requirements = temp_reqs
@@ -1719,6 +1800,7 @@ def do_install(
             if error and traceback:
                 click.echo(crayons.red(error))
                 click.echo(crayons.blue(str(traceback)), err=True)
+                requirements_directory.cleanup()
                 sys.exit(1)
 
     if code:
@@ -1744,24 +1826,55 @@ def do_install(
         # Update project settings with pre preference.
         if pre:
             project.update_settings({'allow_prereleases': pre})
+        if keep_outdated:
+            project.update_settings({'keep_outdated': keep_outdated})
 
-        do_init(dev=dev, allow_global=system, ignore_pipfile=ignore_pipfile, system=system, skip_lock=skip_lock, verbose=verbose, concurrent=concurrent, deploy=deploy, pre=pre)
-
+        do_init(
+            dev=dev, allow_global=system, ignore_pipfile=ignore_pipfile, system=system,
+            skip_lock=skip_lock, verbose=verbose, concurrent=concurrent, deploy=deploy,
+            pre=pre, requirements_dir=requirements_directory
+        )
+        requirements_directory.cleanup()
         sys.exit(0)
 
+    # Support for --selective-upgrade.
+    if selective_upgrade:
+
+        for i, package_name in enumerate(package_names[:]):
+            section = project.packages if not dev else project.dev_packages
+            package = convert_deps_from_pip(package_name)
+            package__name = list(package.keys())[0]
+            package__val = list(package.values())[0]
+
+            try:
+                if not is_star(section[package__name]) and is_star(package__val):
+                    package_names[i] = '{0}{1}'.format(package_name, section[package__name])
+            except KeyError:
+                pass
+
     for package_name in package_names:
         click.echo(crayons.normal(u'Installing {0}…'.format(crayons.green(package_name, bold=True)), bold=True))
 
         # pip install:
         with spinner():
 
-            c = pip_install(package_name, ignore_hashes=True, allow_global=system, no_deps=False, verbose=verbose, pre=pre)
+            c = pip_install(
+                package_name,
+                ignore_hashes=True,
+                allow_global=system,
+                selective_upgrade=selective_upgrade,
+                no_deps=False,
+                verbose=verbose,
+                pre=pre,
+                requirements_dir=requirements_directory.name
+            )
 
             # Warn if --editable wasn't passed.
             try:
                 converted = convert_deps_from_pip(package_name)
             except ValueError as e:
                 click.echo('{0}: {1}'.format(crayons.red('WARNING'), e))
+                requirements_directory.cleanup()
                 sys.exit(1)
 
             key = [k for k in converted.keys()][0]
@@ -1788,6 +1901,7 @@ def do_install(
                     crayons.green(package_name)
                 ), err=True)
             click.echo(crayons.blue(format_pip_error(c.err)), err=True)
+            requirements_directory.cleanup()
             sys.exit(1)
 
         click.echo(
@@ -1809,14 +1923,18 @@ def do_install(
         # Update project settings with pre preference.
         if pre:
             project.update_settings({'allow_prereleases': pre})
+        if keep_outdated:
+            project.update_settings({'keep_outdated': keep_outdated})
 
     if lock and not skip_lock:
-        do_lock(system=system, pre=pre)
+        do_init(dev=dev, allow_global=system, concurrent=concurrent, verbose=verbose, keep_outdated=keep_outdated, requirements_dir=requirements_directory)
+        requirements_directory.cleanup()
 
 
 def do_uninstall(
     package_name=False, more_packages=False, three=None, python=False,
-    system=False, lock=False, all_dev=False, all=False, verbose=False
+    system=False, lock=False, all_dev=False, all=False, verbose=False,
+    keep_outdated=False
 ):
 
     # Automatically use an activated virtualenv.
@@ -1827,7 +1945,8 @@ def do_uninstall(
     ensure_project(three=three, python=python)
 
     # Load the --pre settings from the Pipfile.
-    pre = project.settings.get('pre')
+    pre = project.settings.get('allow_prereleases')
+
 
     package_names = (package_name,) + more_packages
     pipfile_remove = True
@@ -1904,7 +2023,7 @@ def do_uninstall(
             project.remove_package_from_pipfile(package_name, dev=False)
 
     if lock:
-        do_lock(system=system, pre=pre)
+        do_lock(system=system, pre=pre, keep_outdated=keep_outdated)
 
 
 def do_shell(three=None, python=False, fancy=False, shell_args=None):
@@ -2272,7 +2391,7 @@ def do_sync(
     unused=False,
     sequential=False
 ):
-
+    requirements_dir = TemporaryDirectory(suffix='-requirements', prefix='pipenv-')
     # Ensure that virtualenv is available.
     ensure_project(three=three, python=python, validate=False)
 
@@ -2281,7 +2400,8 @@ def do_sync(
     ensure_lockfile()
 
     # Install everything.
-    do_init(dev=dev, verbose=verbose, concurrent=concurrent)
+    do_init(dev=dev, verbose=verbose, concurrent=concurrent, requirements_dir=requirements_dir)
+    requirements_dir.cleanup()
 
     click.echo(
         crayons.green('All dependencies are now up-to-date!')
@@ -2300,13 +2420,15 @@ def do_clean(
     # Ensure that virtualenv is available.
     ensure_project(three=three, python=python, validate=False)
 
+    ensure_lockfile()
+
     installed_packages = delegator.run(
         '{0} freeze'.format(which('pip'))
     ).out.strip().split('\n')
 
     installed_package_names = []
     for installed in installed_packages:
-        r = get_requirement(installed)
+        r = get_requirement(installed, verbose=verbose)
 
         # Ignore editable installations.
         if not r.editable:
@@ -2330,16 +2452,16 @@ def do_clean(
         if used_package in installed_package_names:
             del installed_package_names[installed_package_names.index(used_package)]
 
-    success = False
+    failure = False
     for apparent_bad_package in installed_package_names:
-        success = True
-
         if dry_run:
             click.echo(apparent_bad_package)
         else:
-            click.echo(crayons.white('Unintalling {0}…'.format(repr(apparent_bad_package)), bold=True))
+            click.echo(crayons.white('Uninstalling {0}…'.format(repr(apparent_bad_package)), bold=True))
 
             # Uninstall the package.
-            delegator.run('{0} uninstall {1} -y'.format(which('pip'), apparent_bad_package))
+            c = delegator.run('{0} uninstall {1} -y'.format(which('pip'), apparent_bad_package))
+            if c.return_code != 0:
+                failure = True
 
-    sys.exit(int(success))
\ No newline at end of file
+    sys.exit(int(failure))
