@@ -18,7 +18,11 @@ import warnings
 try:
     from weakref import finalize
 except ImportError:
-    from backports.weakref import finalize
+    try:
+        from backports.weakref import finalize
+    except ImportError:
+        pass
+
 from time import time
 
 logging.basicConfig(level=logging.ERROR)
@@ -30,14 +34,18 @@ except ImportError:
 try:
     from pathlib import Path
 except ImportError:
-    from pathlib2 import Path
+    try:
+        from pathlib2 import Path
+    except ImportError:
+        pass
+
 
 from distutils.spawn import find_executable
 from contextlib import contextmanager
 from piptools.resolver import Resolver
 from piptools.repositories.pypi import PyPIRepository
 from piptools.scripts.compile import get_pip_command
-from piptools import logging
+from piptools import logging as piptools_logging
 from piptools.exceptions import NoCandidateFound
 from pip.download import is_archive_file
 from pip.exceptions import DistributionNotFound
@@ -132,7 +140,7 @@ def get_requirement(dep):
     elif req.local_file and path and not req.vcs:
         req.path = path
         req.uri = None
-    elif req.vcs and req.uri and cleaned_uri and uri != req.uri:
+    elif req.vcs and req.uri and cleaned_uri and cleaned_uri != uri:
         req.uri = strip_ssh_from_git_uri(req.uri)
         req.line = strip_ssh_from_git_uri(req.line)
     req.editable = editable
@@ -267,23 +275,25 @@ def actually_resolve_reps(deps, index_lookup, markers_lookup, project, sources,
 
     req_dir = tempfile.mkdtemp(prefix='pipenv-', suffix='-requirements')
     for dep in deps:
-        if dep.startswith('-e '):
-            constraint = pip.req.InstallRequirement.from_editable(dep[len('-e '):])
-        else:
-            fd, t = tempfile.mkstemp(prefix='pipenv-', suffix='-requirement.txt', dir=req_dir)
-            with os.fdopen(fd, 'w') as f:
-                f.write(dep)
+        if dep:
+            if dep.startswith('-e '):
+                constraint = pip.req.InstallRequirement.from_editable(dep[len('-e '):])
+            else:
+                fd, t = tempfile.mkstemp(prefix='pipenv-', suffix='-requirement.txt', dir=req_dir)
+                with os.fdopen(fd, 'w') as f:
+                    f.write(dep)
 
-            constraint = [c for c in pip.req.parse_requirements(t, session=pip._vendor.requests)][0]
-            # extra_constraints = []
+                constraint = [c for c in pip.req.parse_requirements(t, session=pip._vendor.requests)][0]
 
-        if ' -i ' in dep:
-            index_lookup[constraint.name] = project.get_source(url=dep.split(' -i ')[1]).get('name')
+                # extra_constraints = []
 
-        if constraint.markers:
-            markers_lookup[constraint.name] = str(constraint.markers).replace('"', "'")
+            if ' -i ' in dep:
+                index_lookup[constraint.name] = project.get_source(url=dep.split(' -i ')[1]).get('name')
 
-        constraints.append(constraint)
+            if constraint.markers:
+                markers_lookup[constraint.name] = str(constraint.markers).replace('"', "'")
+
+            constraints.append(constraint)
 
     rmtree(req_dir)
 
@@ -300,10 +310,11 @@ def actually_resolve_reps(deps, index_lookup, markers_lookup, project, sources,
     pip_options, _ = pip_command.parse_args(pip_args)
 
     session = pip_command._build_session(pip_options)
-    pypi = PyPIRepository(pip_options=pip_options, session=session)
+    pypi = PyPIRepository(pip_options=pip_options, use_json=True, session=session)
 
     if verbose:
         logging.log.verbose = True
+        piptools_logging.log.verbose = True
 
 
     resolved_tree = set()
@@ -323,7 +334,7 @@ def actually_resolve_reps(deps, index_lookup, markers_lookup, project, sources,
             ),
             err=True)
 
-        click.echo(crayons.blue(str(e)))
+        click.echo(crayons.blue(str(e)), err=True)
 
         if 'no version found at all' in str(e):
             click.echo(crayons.blue('Please check your version specifier and version number. See PEP440 for more information.'))
@@ -333,7 +344,33 @@ def actually_resolve_reps(deps, index_lookup, markers_lookup, project, sources,
     return resolved_tree, resolver
 
 
-def resolve_deps(deps, which, which_pip, project, sources=None, verbose=False, python=False, clear=False, pre=False, allow_global=False):
+def venv_resolve_deps(deps, which, project, pre=False, verbose=False, clear=False):
+    from . import resolver
+    import json
+
+    resolver = shellquote(resolver.__file__.rstrip('co'))
+    cmd = '{0} {1} {2} {3}'.format(which('python'), resolver, '--pre' if pre else '', '--verbose' if verbose else '')
+    os.environ['PIPENV_PACKAGES'] = '\n'.join(deps)
+
+    c = delegator.run(cmd, block=True)
+
+    del os.environ['PIPENV_PACKAGES']
+
+    try:
+        assert c.return_code == 0
+    except AssertionError:
+        if verbose:
+            click.echo(c.out, err=True)
+        click.echo(c.err[int(len(c.err) / 2) - 1:], err=True)
+        sys.exit(c.return_code)
+
+    if verbose:
+        click.echo(c.out.split('RESULTS:')[0], err=True)
+
+    return json.loads(c.out.split('RESULTS:')[1].strip())
+
+
+def resolve_deps(deps, which, project, sources=None, verbose=False, python=False, clear=False, pre=False, allow_global=False):
     """Given a list of dependencies, return a resolved list of dependencies,
     using pip-tools -- and their hashes, using the warehouse API / pip.
     """
@@ -391,16 +428,19 @@ def resolve_deps(deps, which, which_pip, project, sources=None, verbose=False, p
 
                     for release in cleaned_releases[version]:
                         collected_hashes.append(release['digests']['sha256'])
-
                     collected_hashes = ['sha256:' + s for s in collected_hashes]
 
-                    # Collect un-collectable hashes.
-                    if not collected_hashes:
-                        collected_hashes = list(list(resolver.resolve_hashes([result]).items())[0][1])
-
                 except (ValueError, KeyError, ConnectionError):
                     if verbose:
-                        print('Error fetching {}'.format(name))
+                        click.echo('{0}: Error generating hash for {1}'.format(crayons.red('Warning', bold=True), name))
+
+            # Collect un-collectable hashes (should work with devpi).
+            if not collected_hashes:
+                try:
+                    collected_hashes = list(list(resolver.resolve_hashes([result]).items())[0][1])
+                except (ValueError, KeyError, ConnectionError, IndexError):
+                    if verbose:
+                        print('Error generating hash for {}'.format(name))
 
             d = {'name': name, 'version': version, 'hashes': collected_hashes}
 
@@ -515,15 +555,17 @@ def convert_deps_from_pip(dep):
                 del dependency[key]
     return dependency
 
+def is_star(val):
+    return isinstance(val, six.string_types) and val == '*'
+
+def is_pinned(val):
+    return isinstance(val, six.string_types) and val.startswith('==')
 
 def convert_deps_to_pip(deps, project=None, r=True, include_index=False):
     """"Converts a Pipfile-formatted dependency to a pip-formatted one."""
 
     dependencies = []
 
-    def is_star(val):
-        return isinstance(val, six.string_types) and val == '*'
-
     for dep in deps.keys():
 
         # Default (e.g. '>1.10').
@@ -1085,7 +1127,16 @@ class TemporaryDirectory(object):
     """
 
     def __init__(self, suffix=None, prefix=None, dir=None):
-        self.name = tempfile.mkdtemp(suffix, prefix, dir)
+        if 'RAM_DISK' in os.environ:
+            import uuid
+            name = uuid.uuid4().hex
+            dir_name = os.path.sep.join([os.environ['RAM_DISK'].strip(), name])
+            os.mkdir(dir_name)
+            self.name = dir_name
+
+        else:
+            self.name = tempfile.mkdtemp(suffix, prefix, dir)
+
         self._finalizer = finalize(
             self, self._cleanup, self.name,
             warn_message="Implicitly cleaning up {!r}".format(self))
